local logger = {}
logger.TRACE = -10
logger.DEBUG = 0
logger.INFO = 10
logger.WARNING = 20
logger.ERROR = 30
logger.CRITICAL = 40

logger._names = {}
logger._names[logger.TRACE] = "Trace"
logger._names[logger.DEBUG] = "Debug"
logger._names[logger.INFO] = "Info"
logger._names[logger.WARNING] = "Warning"
logger._names[logger.ERROR] = "Error"
logger._names[logger.CRITICAL] = "Critical"

function logger:new(name)
	local o = {}
	setmetatable(o, self)
	self.__index = self

	self.name = name
	self.prefix = ("[%s] "):format(self.name)
	self.loglevel = logger.TRACE
	return o
end

function logger:is_enabled_for(level)
	return self.loglevel <= level
end

function logger:log(level, message, ...)
	if not self:is_enabled_for(level) then
		return
	end

	local msg = logger:format(level, message, ...)
	core.log("action", msg)
end

function logger:trace(message, ...)
	self:log(logger.TRACE, message, ...)
end

function logger:debug(message, ...)
	self:log(logger.DEBUG, message, ...)
end

function logger:info(message, ...)
	self:log(logger.INFO, message, ...)
end

function logger:warning(message, ...)
	self:log(logger.WARNING, message, ...)
end

function logger:error(message, ...)
	self:log(logger.ERROR, message, ...)
end

function logger:critical(message, ...)
	self:log(logger.CRITICAL, message, ...)
end

function logger:print(level, message, ...)
	if not self:is_enabled_for(level) then
		return
	end

	local msg = self:format(level, message, ...)
	print(msg)
end

function logger:format(level, message, ...)
	local msg = message:format(...)
	local strlevel = ("[%s] "):format(logger._names[level] or "?")
	return self.prefix .. strlevel .. msg
end

watchtower.logger = logger:new("watchtower")
watchtower.logger:trace("Trace message")
watchtower.logger:debug("debug message")
watchtower.logger:info("info message")
watchtower.logger:warning("warning message")
watchtower.logger:error("error message")
watchtower.logger:critical("critical message")
watchtower.logger:print(logger.WARNING, "Print test message")
